//--------------------------------------------------------------
// File     : sps_interpreter.h
//--------------------------------------------------------------

//--------------------------------------------------------------
#ifndef __STM32F4_UB_SPS_INTERPRETER_H
#define __STM32F4_UB_SPS_INTERPRETER_H

//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32f4xx.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_adc.h"
#include "stm32f4xx_dma.h"
#include "stm32_ub_sps.h"
#include "sps_parser.h"
#include "sps_interpreter_oc.h"



//--------------------------------------------------------------
// Flanken Erkennung
//--------------------------------------------------------------
#define  SPS_LO_PEGEL   0x00
#define  SPS_HI_PEGEL   0x01
#define  SPS_LO_FLANKE  0x02
#define  SPS_HI_FLANKE  0x03


//--------------------------------------------------------------
// Cycluszeit und Timerwert
//--------------------------------------------------------------
#define  SPS_MAX_CYCLUS_TIME_US      500  // max 500us
#define  SPS_SOLL_CYCLUS_TIME_MS     1    // 1ms Cycluszeit
volatile uint32_t Systick_Delay;          // Globaler Pausenzaehler
#define  SPS_TIMER_TIC               100  // 100ms


//--------------------------------------------------------------
// Interpreter Error-List
//--------------------------------------------------------------
#define  SPS_ERR_NO_ERROR            0x00  // alles OK
#define  SPS_ERR_UNKNOWN_OPCODE      0x01  // unbekannter Opcode
#define  SPS_ERR_STACK_OVF           0x02  // Stack ueberlauf
#define  SPS_ERR_STACK_UNF           0x03  // Stack unterlauf
#define  SPS_ERR_UNKNOWN_LIST        0x04  // unbekannte Liste
#define  SPS_ERR_UNKNOWN_TYP         0x05  // unbekannter Typ
#define  SPS_ERR_UNKNOWN_PORTPIN     0x06  // Portpin unbekannt
#define  SPS_ERR_HI_CYCLUS_TIME      0x07  // zu hohe cyclus Zeit


//--------------------------------------------------------------
// Struktur fr die SPS Abarbeitung
//--------------------------------------------------------------
typedef struct {
  uint32_t err_nr;
  uint32_t err_pos;
  uint32_t stack_pos;
  uint16_t akt_opcode;
  uint8_t vke_bit; // verknpfungsergebnis
  uint8_t er_bit;  // erstabfrage (0=erstabfrage)
  uint8_t or_bit;
  uint8_t sta_bit;
  uint16_t akku1;
  uint16_t akku2;
  uint16_t akt_liste;
  uint16_t akt_nr;
}SPS_File_t;
SPS_File_t SPS_FILE;


//--------------------------------------------------------------
// Struktur fuer eine Klammerebene
//--------------------------------------------------------------
typedef struct {
  uint16_t opcode;
  uint8_t vke_bit;
  uint8_t or_bit;
}SPS_Stack_t;
SPS_Stack_t SPS_STACK[AWL_MAX_STACK];


//--------------------------------------------------------------
// ADC DR-Register-Adresse
//--------------------------------------------------------------
#define ADC1_DR_ADDRESS    ((uint32_t)0x4001204C)




//--------------------------------------------------------------
// Globale Funktionen
//--------------------------------------------------------------
void Interpreter_Init(void);
uint32_t Interpteter_Test(void);
uint32_t Interpteter_Start(void);
void Interpreter_Stop(void);
void sps_set_status(uint8_t er_bit, uint8_t vke_bit, uint8_t sta_bit, uint8_t or_bit);
void sps_error(uint32_t err_nr);
void sps_update_var(void);
uint8_t sps_read_var_bit(void);
void sps_write_var_bit(uint8_t wert);
void sps_set_var(void);
void sps_reset_var(void);
uint16_t sps_read_var_word(void);
void sps_write_var_word(uint16_t wert);
uint16_t sps_check_var_Flanke(void);
void sps_write_status(uint8_t wert);
uint8_t sps_read_status(void);
void sps_write_timer_bit(uint8_t wert);


//--------------------------------------------------------------
#endif // __STM32F4_UB_SPS_INTERPRETER_H
