//--------------------------------------------------------------
// File     : sps_interpreter_oc.c
// Datum    : 26.02.2014
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F4
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : SPS_Parser
// Funktion : SPS-Interpreter (Ausfhrung)
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "sps_interpreter_oc.h"




//--------------------------------------------------------------
// logisches UND
//--------------------------------------------------------------
void interpreter_OC_Logic_U(void)
{
  uint8_t wert,erg;

  // update von liste und nr
  sps_update_var();

  // wert einlesen
  wert=sps_read_var_bit();

  if(SPS_FILE.er_bit==0) {
    erg=wert;
  }
  else {
    erg=(wert&SPS_FILE.vke_bit);
  }

  erg&=0x01;

  // status (ER, VKE, STA, OR)
  sps_set_status(1,erg,wert,erg);

  // pointer weiterschalten
  SPS_PROG.sps_aktpos+=2;
}


//--------------------------------------------------------------
// logisches NAND
//--------------------------------------------------------------
void interpreter_OC_Logic_UN(void)
{
  uint8_t wert,erg;

  // update von liste und nr
  sps_update_var();

  // wert einlesen
  wert=sps_read_var_bit();

  if(SPS_FILE.er_bit==0) {
    erg=~wert;
  }
  else {
    erg=(~wert&SPS_FILE.vke_bit);
  }

  erg&=0x01;

  // status (ER, VKE, STA, OR)
  sps_set_status(1,erg,wert,erg);

  // pointer weiterschalten
  SPS_PROG.sps_aktpos+=2;
}


//--------------------------------------------------------------
// logisches ODER
//--------------------------------------------------------------
void interpreter_OC_Logic_O(void)
{
  uint8_t wert,erg;

  // update von liste und nr
  sps_update_var();

  // wert einlesen
  wert=sps_read_var_bit();

  if(SPS_FILE.er_bit==0) {
    erg=wert;
  }
  else {
    erg=(wert|SPS_FILE.vke_bit);
  }

  erg&=0x01;

  // status (ER, VKE, STA, OR)
  sps_set_status(1,erg,wert,0);

  // pointer weiterschalten
  SPS_PROG.sps_aktpos+=2;
}


//--------------------------------------------------------------
// logisches NOR
//--------------------------------------------------------------
void interpreter_OC_Logic_ON(void)
{
  uint8_t wert,erg;

  // update von liste und nr
  sps_update_var();

  // wert einlesen
  wert=sps_read_var_bit();

  if(SPS_FILE.er_bit==0) {
    erg=~wert;
  }
  else {
    erg=(~wert|SPS_FILE.vke_bit);
  }

  erg&=0x01;

  // status (ER, VKE, STA, OR)
  sps_set_status(1,erg,wert,0);

  // pointer weiterschalten
  SPS_PROG.sps_aktpos+=2;
}


//--------------------------------------------------------------
// logisches XOR
//--------------------------------------------------------------
void interpreter_OC_Logic_X(void)
{
  uint8_t wert,erg;

  // update von liste und nr
  sps_update_var();

  // wert einlesen
  wert=sps_read_var_bit();

  if(SPS_FILE.er_bit==0) {
    erg=wert;
  }
  else {
    erg=(wert^SPS_FILE.vke_bit);
  }

  erg&=0x01;

  // status (ER, VKE, STA, OR)
  sps_set_status(1,erg,wert,0);

  // pointer weiterschalten
  SPS_PROG.sps_aktpos+=2;
}


//--------------------------------------------------------------
// logisches XNOR
//--------------------------------------------------------------
void interpreter_OC_Logic_XN(void)
{
  uint8_t wert,erg;

  // update von liste und nr
  sps_update_var();

  // wert einlesen
  wert=sps_read_var_bit();

  if(SPS_FILE.er_bit==0) {
    erg=~wert;
  }
  else {
    erg=(~wert^SPS_FILE.vke_bit);
  }

  erg&=0x01;

  // status (ER, VKE, STA, OR)
  sps_set_status(1,erg,wert,0);

  // pointer weiterschalten
  SPS_PROG.sps_aktpos+=2;
}


//--------------------------------------------------------------
// logische Funktion (Verzweigung)
//--------------------------------------------------------------
void interpreter_OC_Verzweigung_On(uint16_t opcode)
{
  SPS_STACK[SPS_FILE.stack_pos].opcode=opcode;
  SPS_STACK[SPS_FILE.stack_pos].or_bit=SPS_FILE.or_bit;
  SPS_STACK[SPS_FILE.stack_pos].vke_bit=SPS_FILE.vke_bit;
  if(SPS_FILE.stack_pos<AWL_MAX_STACK-1) {
    SPS_FILE.stack_pos++;
  }
  else {
    sps_error(SPS_ERR_STACK_OVF);
  }

  // status (ER, VKE, STA, OR)
  sps_set_status(0,99,1,0);
}


//--------------------------------------------------------------
// Verzweigung ENDE
//--------------------------------------------------------------
void interpreter_OC_Verzweigung_Off(void)
{
  uint16_t opcode;
  uint8_t or_bit,vke_bit,erg=0;

  if(SPS_FILE.stack_pos>0) {
    SPS_FILE.stack_pos--;
  }
  else {
    sps_error(SPS_ERR_STACK_UNF);
  }

  opcode=SPS_STACK[SPS_FILE.stack_pos].opcode;
  or_bit=SPS_STACK[SPS_FILE.stack_pos].or_bit;
  vke_bit=SPS_STACK[SPS_FILE.stack_pos].vke_bit;

  if(opcode==AWL_OPCODE_UK) {
    erg=(vke_bit&SPS_FILE.vke_bit)&or_bit;
  }
  else if(opcode==AWL_OPCODE_UNK) {
    erg=~((vke_bit&SPS_FILE.vke_bit)&or_bit);
  }
  else if(opcode==AWL_OPCODE_OK) {
    erg=(vke_bit|SPS_FILE.vke_bit);
  }
  else if(opcode==AWL_OPCODE_ONK) {
    erg=~(vke_bit|SPS_FILE.vke_bit);
  }
  else if(opcode==AWL_OPCODE_XK) {
    erg=(vke_bit^SPS_FILE.vke_bit);
  }
  else if(opcode==AWL_OPCODE_XNK) {
    erg=~(vke_bit^SPS_FILE.vke_bit);
  }
  else {
    sps_error(SPS_ERR_UNKNOWN_OPCODE);
  }

  erg&=0x01;

  // status (ER, VKE, STA, OR)
  sps_set_status(1,erg,1,or_bit);
}


//--------------------------------------------------------------
// Zuweisung
//--------------------------------------------------------------
void interpreter_OC_EQUAL(void)
{
  // update von liste und nr
  sps_update_var();

  // wert schreiben
  sps_write_var_bit(SPS_FILE.vke_bit);

  // status (ER, VKE, STA, OR)
  sps_set_status(0,99,SPS_FILE.vke_bit,0);

  // pointer weiterschalten
  SPS_PROG.sps_aktpos+=2;
}


//--------------------------------------------------------------
// Reset/Set von einem Bit (oder Wert)
//--------------------------------------------------------------
void interpreter_OC_RS(uint16_t opcode)
{
  // update von liste und nr
  sps_update_var();

  if(SPS_FILE.vke_bit==1) {
    if(opcode==AWL_OPCODE_R) {
      // bit ruecksetzen
      sps_reset_var();
    }
    else if(opcode==AWL_OPCODE_S) {
      // bit setzen
      sps_set_var();
    }
    else {
      sps_error(SPS_ERR_UNKNOWN_OPCODE);
    }
  }

  // status (ER, VKE, STA, OR)
  sps_set_status(0,99,99,0);

  // pointer weiterschalten
  SPS_PROG.sps_aktpos+=2;
}


//--------------------------------------------------------------
// negiere VKE
//--------------------------------------------------------------
void interpreter_OC_NOT(void)
{
  uint8_t erg=0;

  if(SPS_FILE.vke_bit==0) {
    erg=1;
  }
  else {
    erg=0;
  }

  // status (ER, VKE, STA, OR)
  sps_set_status(99,erg,1,99);
}


//--------------------------------------------------------------
// setze VKE
//--------------------------------------------------------------
void interpreter_OC_SET(void)
{
  // status (ER, VKE, STA, OR)
  sps_set_status(0,1,1,0);
}


//--------------------------------------------------------------
// loesche VKE
//--------------------------------------------------------------
void interpreter_OC_CLR(void)
{
  // status (ER, VKE, STA, OR)
  sps_set_status(0,0,0,0);
}


//--------------------------------------------------------------
// Flanke negativ
//--------------------------------------------------------------
void interpreter_OC_FN(void)
{
  uint16_t wert;
  uint8_t erg=0;

  // update von liste und nr
  sps_update_var();

  // aktuellen VKE Wert schreiben (um flanke zu erkennen)
  sps_write_var_bit(SPS_FILE.vke_bit);

  // check auf Lo-Flanke
  wert=sps_check_var_Flanke();
  if(wert==SPS_LO_FLANKE) {
    erg=1;
  }

  // status (ER, VKE, STA, OR)
  sps_set_status(1,erg,SPS_FILE.vke_bit,0);

  // pointer weiterschalten
  SPS_PROG.sps_aktpos+=2;
}


//--------------------------------------------------------------
// Flanke positiv
//--------------------------------------------------------------
void interpreter_OC_FP(void)
{
  uint16_t wert;
  uint8_t erg=0;

  // update von liste und nr
  sps_update_var();

  // aktuellen VKE Wert schreiben (um flanke zu erkennen)
  sps_write_var_bit(SPS_FILE.vke_bit);

  // check auf Hi-Flanke
  wert=sps_check_var_Flanke();
  if(wert==SPS_HI_FLANKE) {
    erg=1;
  }

  // status (ER, VKE, STA, OR)
  sps_set_status(1,erg,SPS_FILE.vke_bit,0);

  // pointer weiterschalten
  SPS_PROG.sps_aktpos+=2;
}


//--------------------------------------------------------------
// Vergleich
//--------------------------------------------------------------
void interpreter_OC_Vergleich(uint16_t opcode)
{
  uint8_t erg=0;

  if(opcode==AWL_OPCODE_VEQ) {
    if(SPS_FILE.akku2==SPS_FILE.akku1) erg=1;
  }
  else if(opcode==AWL_OPCODE_VGK) {
    if(SPS_FILE.akku2>SPS_FILE.akku1) erg=1;
    if(SPS_FILE.akku2<SPS_FILE.akku1) erg=1;
  }
  else if(opcode==AWL_OPCODE_VG) {
    if(SPS_FILE.akku2>SPS_FILE.akku1) erg=1;
  }
  else if(opcode==AWL_OPCODE_VK) {
    if(SPS_FILE.akku2<SPS_FILE.akku1) erg=1;
  }
  else if(opcode==AWL_OPCODE_VGG) {
    if(SPS_FILE.akku2>=SPS_FILE.akku1) erg=1;
  }
  else if(opcode==AWL_OPCODE_VKG) {
    if(SPS_FILE.akku2<=SPS_FILE.akku1) erg=1;
  }
  else {
    sps_error(SPS_ERR_UNKNOWN_OPCODE);
  }

  // status (ER, VKE, STA, OR)
  sps_set_status(1,erg,SPS_FILE.vke_bit,0);
}


//--------------------------------------------------------------
// Rechne
//--------------------------------------------------------------
void interpreter_OC_Rechne(uint16_t opcode)
{
  uint16_t erg=0;

  if(opcode==AW_OPCODE_RA) {
    erg=SPS_FILE.akku2+SPS_FILE.akku1;
    SPS_FILE.akku1=erg;
  }
  else if(opcode==AW_OPCODE_RS) {
    erg=SPS_FILE.akku2-SPS_FILE.akku1;
    SPS_FILE.akku1=erg;
  }
  else if(opcode==AW_OPCODE_RM) {
    erg=SPS_FILE.akku2*SPS_FILE.akku1;
    SPS_FILE.akku1=erg;
  }
  else if(opcode==AW_OPCODE_RD) {
    erg=SPS_FILE.akku2/SPS_FILE.akku1;
    SPS_FILE.akku1=erg;
  }
  else {
    sps_error(SPS_ERR_UNKNOWN_OPCODE);
  }
}


//--------------------------------------------------------------
// Load von einem Memory Wert in Akku1
// (alter Wert von Akku1 wird in Akku2 gesichert)
//--------------------------------------------------------------
void interpreter_OC_Load_M(void)
{
  uint16_t wert;

  // update von liste und nr
  sps_update_var();

  // wert einlesen
  wert=sps_read_var_word();

  // wert in Akku speichern
  SPS_FILE.akku2=SPS_FILE.akku1;
  SPS_FILE.akku1=wert;

  // pointer weiterschalten
  SPS_PROG.sps_aktpos+=2;
}


//--------------------------------------------------------------
// Load von einem Konstanten Wert in Akku1
// (alter Wert von Akku1 wird in Akku2 gesichert)
//--------------------------------------------------------------
void interpreter_OC_Load_C(void)
{
  uint16_t wert;

  // Konstante lesen
  SPS_PROG.sps_aktpos++;
  wert=SPS_PROG.bytecode[SPS_PROG.sps_aktpos];

  // wert in Akku speichern
  SPS_FILE.akku2=SPS_FILE.akku1;
  SPS_FILE.akku1=wert;
}


//--------------------------------------------------------------
// Transfer vom Akku1 in eine Word Variable
//--------------------------------------------------------------
void interpreter_OC_Transfer(void)
{
  uint16_t wert;

  // update von liste und nr
  sps_update_var();

  wert=SPS_FILE.akku1;
  // akku in Variable speichern
  sps_write_var_word(wert);

  // pointer weiterschalten
  SPS_PROG.sps_aktpos+=2;
}


//--------------------------------------------------------------
// Zaehler (vorwaerts)
//--------------------------------------------------------------
void interpreter_OC_Counter_ZV(void)
{
  uint16_t wert;

  // update von liste und nr
  sps_update_var();

  // aktuellen VKE Wert schreiben (um flanke zu erkennen)
  sps_write_var_bit(SPS_FILE.vke_bit);

  // check auf Hi-Flanke
  wert=sps_check_var_Flanke();
  if(wert==SPS_HI_FLANKE) {
    // wert einlesen
    wert=sps_read_var_word();
    if(wert<65535) {
      // hochzaehlen
      wert++;
      // neuen Wert speichern
      sps_write_var_word(wert);
    }
  }

  // status (ER, VKE, STA, OR)
  sps_set_status(0,99,99,0);

  // pointer weiterschalten
  SPS_PROG.sps_aktpos+=2;
}


//--------------------------------------------------------------
// Zaehler (rueckwaerts)
//--------------------------------------------------------------
void interpreter_OC_Counter_ZR(void)
{
  uint16_t wert;

  // update von liste und nr
  sps_update_var();

  // aktuellen VKE Wert schreiben (um flanke zu erkennen)
  sps_write_var_bit(SPS_FILE.vke_bit);

  // check auf Hi-Flanke
  wert=sps_check_var_Flanke();
  if(wert==SPS_HI_FLANKE) {
    // wert einlesen
    wert=sps_read_var_word();
    if(wert>0) {
      // runterzaehlen
      wert--;
      // neuen Wert speichern
      sps_write_var_word(wert);
    }
  }

  // status (ER, VKE, STA, OR)
  sps_set_status(0,99,99,0);

  // pointer weiterschalten
  SPS_PROG.sps_aktpos+=2;
}


//--------------------------------------------------------------
// Timer (Impuls)
//--------------------------------------------------------------
void interpreter_OC_Timer_SI(void)
{
  uint16_t wert;

  // update von liste und nr
  sps_update_var();

  // aktuellen VKE Wert schreiben (um flanke zu erkennen)
  sps_write_var_bit(SPS_FILE.vke_bit);

  // check welche Flanke
  wert=sps_check_var_Flanke();
  if(wert==SPS_HI_FLANKE) {
    // Ausgang = HI
    sps_write_timer_bit(1);
    // Timer starten
    sps_write_status(SPS_TIM_STATUS_START);
  }
  else if(wert==SPS_LO_FLANKE) {
    // Ausgang = LO
    sps_write_timer_bit(0);
    // Timer stoppen
    sps_write_status(SPS_TIM_STATUS_STOP);
  }
  else {
    // check ob timer abgelaufen
    if(sps_read_status()==SPS_TIM_STATUS_TIMEOUT) {
      // Ausgang = LO
      sps_write_timer_bit(0);
      // Timer stoppen
      sps_write_status(SPS_TIM_STATUS_STOP);
    }
  }

  // status (ER, VKE, STA, OR)
  sps_set_status(0,99,99,0);

  // pointer weiterschalten
  SPS_PROG.sps_aktpos+=2;
}


//--------------------------------------------------------------
// Timer (verlngerter Impuls)
//--------------------------------------------------------------
void interpreter_OC_Timer_SV(void)
{
  uint16_t wert;

  // update von liste und nr
  sps_update_var();

  // aktuellen VKE Wert schreiben (um flanke zu erkennen)
  sps_write_var_bit(SPS_FILE.vke_bit);

  // check welche Flanke
  wert=sps_check_var_Flanke();
  if(wert==SPS_HI_FLANKE) {
    // Ausgang = HI
    sps_write_timer_bit(1);
    // Timer starten
    sps_write_status(SPS_TIM_STATUS_START);
  }
  else {
    // check ob timer abgelaufen
    if(sps_read_status()==SPS_TIM_STATUS_TIMEOUT) {
      // Ausgang = LO
      sps_write_timer_bit(0);
      // Timer stoppen
     sps_write_status(SPS_TIM_STATUS_STOP);
    }
  }

  // status (ER, VKE, STA, OR)
  sps_set_status(0,99,99,0);

  // pointer weiterschalten
  SPS_PROG.sps_aktpos+=2;
}


//--------------------------------------------------------------
// Timer (einschaltverzgerung)
//--------------------------------------------------------------
void interpreter_OC_Timer_SE(void)
{
  uint16_t wert;

  // update von liste und nr
  sps_update_var();

  // aktuellen VKE Wert schreiben (um flanke zu erkennen)
  sps_write_var_bit(SPS_FILE.vke_bit);

  // check welche Flanke
  wert=sps_check_var_Flanke();
  if(wert==SPS_HI_FLANKE) {
    // Timer starten
    sps_write_status(SPS_TIM_STATUS_START);
  }
  else if(wert==SPS_LO_FLANKE) {
    // Ausgang = LO
    sps_write_timer_bit(0);
    // Timer stoppen
    sps_write_status(SPS_TIM_STATUS_STOP);
  }
  else {
    // check ob timer abgelaufen
    if(sps_read_status()==SPS_TIM_STATUS_TIMEOUT) {
      // Ausgang = HI
      sps_write_timer_bit(1);
      // Timer stoppen
      sps_write_status(SPS_TIM_STATUS_STOP);
    }
  }

  // status (ER, VKE, STA, OR)
  sps_set_status(0,99,99,0);

  // pointer weiterschalten
  SPS_PROG.sps_aktpos+=2;
}


//--------------------------------------------------------------
// Timer (speichernde einschaltverzgerung)
//--------------------------------------------------------------
void interpreter_OC_Timer_SS(void)
{
  uint16_t wert;

  // update von liste und nr
  sps_update_var();

  // aktuellen VKE Wert schreiben (um flanke zu erkennen)
  sps_write_var_bit(SPS_FILE.vke_bit);

  // check welche Flanke
  wert=sps_check_var_Flanke();
  if(wert==SPS_HI_FLANKE) {
    // Timer starten
    sps_write_status(SPS_TIM_STATUS_START);
  }
  else {
    // check ob timer abgelaufen
    if(sps_read_status()==SPS_TIM_STATUS_TIMEOUT) {
      // Ausgang = HI
      sps_write_timer_bit(1);
      // Timer stoppen
      sps_write_status(SPS_TIM_STATUS_STOP);
    }
  }

  // status (ER, VKE, STA, OR)
  sps_set_status(0,99,99,0);

  // pointer weiterschalten
  SPS_PROG.sps_aktpos+=2;
}


//--------------------------------------------------------------
// Timer (ausschaltverzgerung)
//--------------------------------------------------------------
void interpreter_OC_Timer_SA(void)
{
  uint16_t wert;

  // update von liste und nr
  sps_update_var();

  // aktuellen VKE Wert schreiben (um flanke zu erkennen)
  sps_write_var_bit(SPS_FILE.vke_bit);

  // check welche Flanke
  wert=sps_check_var_Flanke();
  if(wert==SPS_HI_FLANKE) {
    // Ausgang = HI
    sps_write_timer_bit(1);
  }
  else if(wert==SPS_LO_FLANKE) {
    // Timer starten
    sps_write_status(SPS_TIM_STATUS_START);
  }
  else {
    // check ob timer abgelaufen
    if(sps_read_status()==SPS_TIM_STATUS_TIMEOUT) {
      // Ausgang = LO
      sps_write_timer_bit(0);
      // Timer stoppen
      sps_write_status(SPS_TIM_STATUS_STOP);
    }
  }

  // status (ER, VKE, STA, OR)
  sps_set_status(0,99,99,0);

  // pointer weiterschalten
  SPS_PROG.sps_aktpos+=2;
}
