//--------------------------------------------------------------
// File     : sps_parser.c
// Datum    : 26.02.2014
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F4
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : retarget_printf
// Funktion : SPS-Parser
//            Erstellt aus dem AWL-File eine Opcode-Liste
//            fr den Interpreter
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "sps_parser.h"



//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
void p_parser_init(void);
uint32_t checkAWL(void);
uint32_t getNextToken(void);
uint32_t copyNextToken(void);
void skipLine(void);
void checkKommand(void);
void TYP_VAR_INPUT(void);
void TYP_VAR_OUTPUT(void);
void TYP_VAR_MEMORY(void);
void TYP_VAR_ADC(void);
void TYP_AWL_START(void);
void TYP_AWL_END(void);
uint32_t checkPortPin(void);
uint32_t checkAdcPin(void);
uint32_t checkVarTyp(void);
int32_t dezStringToInt(uint32_t len);



//--------------------------------------------------------------
// parst komplette AWL
// ret_wert : 0=ok, >0=fehler-nr
//--------------------------------------------------------------
uint32_t Parser_Start(void)
{
  uint32_t ret_wert=0;
  uint32_t l;

  // init aller Variablen
  p_parser_init();

  // check ob alle Zeichen = Ascii
  AWL_ERR.err_nr=checkAWL();
  if(AWL_ERR.err_nr==0) {
    // ok
    AWL_FILE.awl_flags|=AWL_FLAG_CHECK_OK;
    // alle Tokens der AWL parsen
    do {
      l=getNextToken();
      if(l>0) checkKommand();
      if(SPS_PROG.sps_size>=SPS_MAX_SIZE ) {
        Parser_setError(AWL_ERR_OPCODE_MAX);
      }
    }while((AWL_FILE.awl_aktpos<AWL_FILE.awl_akt_size) && (AWL_ERR.err_nr==0));
  }

  ret_wert=AWL_ERR.err_nr;

  // check ob parsing bis jetzt ok
  if(ret_wert==0) {
    // check ob Stack ausgeglichen
    if(AWL_FILE.awl_akt_stack!=0) {
      Parser_setError(AWL_ERR_STACK_UNEQUAL);
      ret_wert=AWL_ERR.err_nr;
    }
    else {
      AWL_FILE.awl_flags|=AWL_FLAG_PARSE_OK;

      // check ob ein programm erstellt wurde
      if(SPS_PROG.sps_size>0) {
        AWL_FILE.awl_flags|=AWL_FLAG_PRG_OK;
      }

      if(AWL_FILE.awl_flags!=AWL_FLAG_ALL_OK) {
        Parser_setError(AWL_ERR_FLAG_ERROR);
        ret_wert=AWL_ERR.err_nr;
      }
    }
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// fehlernummer abspeichern
//--------------------------------------------------------------
void Parser_setError(uint32_t error)
{
  if(AWL_ERR.err_nr==0) {
    AWL_ERR.err_nr=error;
    AWL_ERR.err_line=AWL_FILE.awl_akt_line;
    strcpy(AWL_ERR.err_token,AWL_FILE.awl_token);
  }
}


//--------------------------------------------------------------
// init aller Parser Variablen
//--------------------------------------------------------------
void p_parser_init(void)
{
  // init aller Error Variabeln
  AWL_ERR.err_line=0;
  AWL_ERR.err_nr=0;
  AWL_ERR.err_token[0]=0x00;
  // init aller AWL Variablen
  AWL_FILE.awl_akt_size=0;
  AWL_FILE.awl_aktpos=0;
  AWL_FILE.awl_akt_line=1; // start = zeile 1
  AWL_FILE.awl_akt_stack=0;
  AWL_FILE.awl_flags=0;
  AWL_FILE.awl_abschnitt=0;
  AWL_FILE.awl_token[0]=0x00;
  // init aller Variablen
  VAR_INPUT.anz=0;
  VAR_OUTPUT.anz=0;
  VAR_MEMORY.anz=0;
  VAR_ADC.anz=0;
  // init aller SPS Variabeln
  SPS_PROG.sps_size=0;
  SPS_PROG.sps_aktpos=0;
}


//--------------------------------------------------------------
// prueft komplette AWL ob nur Ascii-Zeichen benutzt wurden
// ret_wert : 0=ok, >0=fehler-nr
//--------------------------------------------------------------
uint32_t checkAWL(void)
{
  uint32_t ret_wert=0;
  uint32_t n;
  uint8_t c;

  if(AWL_GLOBAL.awl_quelle==0) {
    AWL_FILE.awl_akt_size=AWL_GLOBAL.awl_flash_size;
  }
  else {
    AWL_FILE.awl_akt_size=AWL_GLOBAL.awl_ram_size;
  }

  if(AWL_FILE.awl_akt_size==0) return(AWL_ERR_FILESIZE_0); // laenge falsch

  // check character (muss ascii sein)
  AWL_ERR.err_line=1;
  for(n=0;n<AWL_FILE.awl_akt_size;n++) {
    if(AWL_GLOBAL.awl_quelle==0) {
      c=awl_struct.table[n];
    }
    else {
      c=AWL_BUFFER[n];
    }
    if((c<32) || (c>127)) {
      if(c==10) AWL_ERR.err_line++;
      if((c!=10) && (c!=13)) {
        ret_wert=AWL_ERR_NO_ASCII; // falsches zeichen
        break;
      }
    }
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// sucht naechstes Token vom AWL (Kommentare werden bersprungen)
// (ab aktueller position)
// ret_wert = laenge vom string (0=kein string)
//  Teilstring steht in "AWL_FILE.awl_token[]"
//--------------------------------------------------------------
uint32_t getNextToken(void)
{
  uint32_t ret_wert=0;
  uint32_t l;

  do {
    l=copyNextToken();
    if(l>0) {
      // check auf kommentar
      if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_REM)==0) {
        // zeile ueberspringen
        skipLine();
      }
      else {
        // token gefunden
        ret_wert=l;
        break;
      }
    }
  }while((AWL_FILE.awl_aktpos<AWL_FILE.awl_akt_size) && (AWL_ERR.err_nr==0)) ;

  return(ret_wert);
}


//--------------------------------------------------------------
// kopiert einen Teilstring vom AWL
// (ab aktueller position)
// ret_wert = laenge vom string (0=kein string)
//  Teilstring steht in "AWL_FILE.awl_token[]"
//--------------------------------------------------------------
uint32_t copyNextToken(void)
{
  uint32_t ret_wert=0;
  int32_t start,ende;  // muss signed int sein !!
  uint32_t n,m;
  uint8_t c;

  AWL_FILE.awl_token[0]=0x00;
  start=-1;
  ende=-1;
  do {
    if(AWL_GLOBAL.awl_quelle==0) {
      c=awl_struct.table[AWL_FILE.awl_aktpos];
    }
    else {
      c=AWL_BUFFER[AWL_FILE.awl_aktpos];
    }
    if(c==10) {
      AWL_FILE.awl_akt_line++;
    }

    if(start==-1) {
      if((c!=32) && (c!=10) && (c!=13)) {
        start=AWL_FILE.awl_aktpos;
      }
    }
    else if(ende==-1) {
      if((c==32) || (c==10) || (c==13)) {
        ende=AWL_FILE.awl_aktpos;
        break;
      }
    }
    AWL_FILE.awl_aktpos++;
  }while(AWL_FILE.awl_aktpos<AWL_FILE.awl_akt_size);

  if((start!=-1) && (ende!=-1)) {
    if(ende>start) {
      ret_wert=(ende-start);
      if(ret_wert>=AWL_MAX_TOKEN_LEN) {
        // Token ist zu lange
        ret_wert=0;
        m=0;
        for(n=start;n<start+AWL_MAX_TOKEN_LEN-1;n++) {
          if(AWL_GLOBAL.awl_quelle==0) {
            AWL_FILE.awl_token[m]=awl_struct.table[n];
          }
          else {
            AWL_FILE.awl_token[m]=AWL_BUFFER[n];
          }
          m++;
        }
        AWL_FILE.awl_token[m]=0x00;
        Parser_setError(AWL_ERR_TOKEN_LEN);
      }
      else {
        // Token ist ok
        m=0;
        for(n=start;n<ende;n++) {
          if(AWL_GLOBAL.awl_quelle==0) {
            AWL_FILE.awl_token[m]=awl_struct.table[n];
          }
          else {
            AWL_FILE.awl_token[m]=AWL_BUFFER[n];
          }
          m++;
        }
        AWL_FILE.awl_token[m]=0x00;
      }
    }
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// ueberspringt eine Zeile bis zum LineFeed = 0x0A
//--------------------------------------------------------------
void skipLine(void)
{
  uint8_t c;

  do {
    if(AWL_GLOBAL.awl_quelle==0) {
      c=awl_struct.table[AWL_FILE.awl_aktpos];
    }
    else {
      c=AWL_BUFFER[AWL_FILE.awl_aktpos];
    }

    if(c==10) {
      AWL_FILE.awl_akt_line++;
      AWL_FILE.awl_aktpos++;
      break;
    }
    AWL_FILE.awl_aktpos++;
  }while(AWL_FILE.awl_aktpos<AWL_FILE.awl_akt_size);
}


//--------------------------------------------------------------
// aufteilung nach kommando
// falls kommando unbekannt -> AWL_ERR.err_nr>0
//--------------------------------------------------------------
void checkKommand(void)
{
  if(AWL_FILE.awl_abschnitt==0) {
    //---------------------------
    // Allgemeiner Bereich
    //---------------------------
    if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_VAR_INPUT)==0) {
      TYP_VAR_INPUT();
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_VAR_OUTPUT)==0) {
      TYP_VAR_OUTPUT();
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_VAR_MEMORY)==0) {
      TYP_VAR_MEMORY();
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_VAR_ADC)==0) {
      TYP_VAR_ADC();
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_AWL_START)==0) {
      TYP_AWL_START();
    }
    else {
      Parser_setError(AWL_ERR_UNKNOWN_TOKEN);
    }
  }
  else {
    //---------------------------
    // Programm Bereich
    //---------------------------
    if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_AWL_END)==0) {
      TYP_AWL_END();
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_U)==0) {
      parser_OC_Logic(AWL_OPCODE_U);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_UK)==0) {
      parser_OC_Logic_Verzweigung_On(AWL_OPCODE_UK);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_UN)==0) {
      parser_OC_Logic(AWL_OPCODE_UN);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_UNK)==0) {
      parser_OC_Logic_Verzweigung_On(AWL_OPCODE_UNK);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_O)==0) {
      parser_OC_Logic(AWL_OPCODE_O);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_OK)==0) {
      parser_OC_Logic_Verzweigung_On(AWL_OPCODE_OK);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_ON)==0) {
      parser_OC_Logic(AWL_OPCODE_ON);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_ONK)==0) {
      parser_OC_Logic_Verzweigung_On(AWL_OPCODE_ONK);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_X)==0) {
      parser_OC_Logic(AWL_OPCODE_X);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_XK)==0) {
      parser_OC_Logic_Verzweigung_On(AWL_OPCODE_XK);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_XN)==0) {
      parser_OC_Logic(AWL_OPCODE_XN);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_XNK)==0) {
      parser_OC_Logic_Verzweigung_On(AWL_OPCODE_XNK);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_KE)==0) {
      parser_OC_Logic_Verzweigung_Off(AWL_OPCODE_KE);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_EQ)==0) {
      parser_OC_EQUAL();
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_R)==0) {
      parser_OC_RS(AWL_OPCODE_R);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_S)==0) {
      parser_OC_RS(AWL_OPCODE_S);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_NOT)==0) {
      parser_OC_NSC(AWL_OPCODE_NOT);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_SET)==0) {
      parser_OC_NSC(AWL_OPCODE_SET);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_CLR)==0) {
      parser_OC_NSC(AWL_OPCODE_CLR);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_FN)==0) {
      parser_OC_Flanke(AWL_OPCODE_FN);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_FP)==0) {
      parser_OC_Flanke(AWL_OPCODE_FP);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_VEQ)==0) {
      parser_OC_Vergleich(AWL_OPCODE_VEQ);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_VGK)==0) {
      parser_OC_Vergleich(AWL_OPCODE_VGK);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_VG)==0) {
      parser_OC_Vergleich(AWL_OPCODE_VG);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_VK)==0) {
      parser_OC_Vergleich(AWL_OPCODE_VK);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_VGG)==0) {
      parser_OC_Vergleich(AWL_OPCODE_VGG);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_VKG)==0) {
      parser_OC_Vergleich(AWL_OPCODE_VKG);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_RA)==0) {
      parser_OC_Rechne(AW_OPCODE_RA);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_RS)==0) {
      parser_OC_Rechne(AW_OPCODE_RS);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_RM)==0) {
      parser_OC_Rechne(AW_OPCODE_RM);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_RD)==0) {
      parser_OC_Rechne(AW_OPCODE_RD);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_L)==0) {
      parser_OC_Load();
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_T)==0) {
      parser_OC_Transfer();
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_ZV)==0) {
      parser_OC_Counter(AWL_OPCODE_ZV);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_ZR)==0) {
      parser_OC_Counter(AWL_OPCODE_ZR);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_SI)==0) {
      parser_OC_Timer(AWL_OPCODE_SI);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_SV)==0) {
      parser_OC_Timer(AWL_OPCODE_SV);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_SE)==0) {
      parser_OC_Timer(AWL_OPCODE_SE);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_SS)==0) {
      parser_OC_Timer(AWL_OPCODE_SS);
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_SA)==0) {
      parser_OC_Timer(AWL_OPCODE_SA);
    }
    else {
      Parser_setError(AWL_ERR_UNKNOWN_TOKEN);
    }
  }
}


//--------------------------------------------------------------
// TYP = Var Input
//
// VAR_INPUT
//   S1 AT PB13
//   Schalter2 AT PC4
// VAR_END
//--------------------------------------------------------------
void TYP_VAR_INPUT(void)
{
  uint32_t l,ok=0;

  do {
    // variablenname (oder ENDE)
    l=getNextToken();
    if(l>0) {
      if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_VAR_END)==0) {
        ok=1; // fertig
      }
      else {
        strcpy(VAR_INPUT.token[VAR_INPUT.anz],AWL_FILE.awl_token);

        // at
        l=getNextToken();
        if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_VAR_AT)==0) {
          // Port
          l=checkPortPin();
          if(l>0) {
            VAR_INPUT.address[VAR_INPUT.anz]=l;
            VAR_INPUT.anz++;
            if(VAR_INPUT.anz>AWL_MAX_VAR_IN) {
              Parser_setError(AWL_ERR_VAR_IN_MAX);
              ok=2; // err
            }
          }
          else {
            ok=2; // err
          }
        }
        else {
          Parser_setError(AWL_ERR_AT_MISSING);
          ok=2; // err
        }
      }
    }
  }
  while((AWL_FILE.awl_aktpos<AWL_FILE.awl_akt_size) && (ok==0));
}


//--------------------------------------------------------------
// TYP = Var Output
//
// VAR_OUTPUT
//   LED1 AT PB13
//   Motor AT PC4
// VAR_END
//--------------------------------------------------------------
void TYP_VAR_OUTPUT(void)
{
  uint32_t l,ok=0;

  do {
    // variablenname (oder ENDE)
    l=getNextToken();
    if(l>0) {
      if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_VAR_END)==0) {
        ok=1; // fertig
      }
      else {
        strcpy(VAR_OUTPUT.token[VAR_OUTPUT.anz],AWL_FILE.awl_token);

        // at
        l=getNextToken();
        if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_VAR_AT)==0) {
          // Port
          l=checkPortPin();
          if(l>0) {
            VAR_OUTPUT.address[VAR_OUTPUT.anz]=l;
            VAR_OUTPUT.anz++;
            if(VAR_OUTPUT.anz>AWL_MAX_VAR_OUT) {
              Parser_setError(AWL_ERR_VAR_OUT_MAX);
              ok=2; // err
            }
          }
          else {
            ok=2; // err
          }
        }
        else {
          Parser_setError(AWL_ERR_AT_MISSING);
          ok=2; // err
        }
      }
    }
  }
  while((AWL_FILE.awl_aktpos<AWL_FILE.awl_akt_size) && (ok==0));

  if(VAR_OUTPUT.anz>0) {
    AWL_FILE.awl_flags|=AWL_FLAG_VAR_OUT;
  }
}


//--------------------------------------------------------------
// TYP = Var Memory
//
// VAR
//   Merker1 : BOOL
//   IC1 : TIMER
// VAR_END
//--------------------------------------------------------------
void TYP_VAR_MEMORY(void)
{
  uint32_t l,ok=0;

  do {
    // variablenname (oder ENDE)
    l=getNextToken();
    if(l>0) {
      if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_VAR_END)==0) {
        ok=1; // fertig
      }
      else {
        strcpy(VAR_MEMORY.token[VAR_MEMORY.anz],AWL_FILE.awl_token);

        // :
        l=getNextToken();
        if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_VAR_DP)==0) {
          // Typ
          l=checkVarTyp();
          if(l!=SPS_TYP_NONE) {
            VAR_MEMORY.typ[VAR_MEMORY.anz]=l;
            VAR_MEMORY.anz++;
            if(VAR_MEMORY.anz>AWL_MAX_VAR_MEMORY) {
              Parser_setError(AWL_ERR_VAR_MEM_MAX);
              ok=2; // err
            }
          }
          else {
            ok=2; // err
          }
        }
        else {
          Parser_setError(AWL_ERR_DP_MISSING);
          ok=2; // err
        }
      }
    }
  }
  while((AWL_FILE.awl_aktpos<AWL_FILE.awl_akt_size) && (ok==0));
}


//--------------------------------------------------------------
// TYP = Var ADC
//
// VAR_ADC
//   Sensor AT PA5
// VAR_END
//--------------------------------------------------------------
void TYP_VAR_ADC(void)
{
  uint32_t l,ok=0;

  do {
    // variablenname (oder ENDE)
    l=getNextToken();
    if(l>0) {
      if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_VAR_END)==0) {
        ok=1; // fertig
      }
      else {
        strcpy(VAR_ADC.token[VAR_ADC.anz],AWL_FILE.awl_token);

        // at
        l=getNextToken();
        if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_VAR_AT)==0) {
          // Port
          l=checkAdcPin();
          if(l>0) {
            VAR_ADC.address[VAR_ADC.anz]=l;
            VAR_ADC.anz++;
            if(VAR_ADC.anz>AWL_MAX_VAR_ADC) {
              Parser_setError(AWL_ERR_VAR_ADC_MAX);
              ok=2; // err
            }
          }
          else {
            ok=2; // err
          }
        }
        else {
          Parser_setError(AWL_ERR_AT_MISSING);
          ok=2; // err
        }
      }
    }
  }
  while((AWL_FILE.awl_aktpos<AWL_FILE.awl_akt_size) && (ok==0));
}


//--------------------------------------------------------------
// TYP = AWL START
//--------------------------------------------------------------
void TYP_AWL_START(void)
{
  if((AWL_FILE.awl_flags&AWL_FLAG_VAR_OUT)==0) {
    Parser_setError(AWL_ERR_VAR_OUT_MISSING);
  }
  else {
    AWL_FILE.awl_flags|=AWL_FLAG_START;
    // Abschnitt auf Programm umschalten
    AWL_FILE.awl_abschnitt=1;
  }
}


//--------------------------------------------------------------
// TYP = AWL END
//--------------------------------------------------------------
void TYP_AWL_END(void)
{
  if((AWL_FILE.awl_flags&AWL_FLAG_START)==0) {
    Parser_setError(AWL_ERR_START_MISSING);
  }
  else {
    AWL_FILE.awl_flags|=AWL_FLAG_END;
    // Abschnitt auf Allgemein umschalten
    AWL_FILE.awl_abschnitt=0;
  }
}


//--------------------------------------------------------------
// Auswertung vom PortPin Token
// ret_wert : 0 = error
//            [PA0] = 0x0101 bis [PA15] = 0x0110
//            [PB0] = 0x0201 bis [PB15] = 0x0210
//            [PG0] = 0x0701 bis [PG15] = 0x0710
//--------------------------------------------------------------
uint32_t checkPortPin(void)
{
  uint32_t ret_wert=0;
  uint32_t l;

  l=getNextToken();
  if(l==3) {
    // z.B. PA3
    if(AWL_FILE.awl_token[1]=='A') ret_wert=SPS_PORTA;
    if(AWL_FILE.awl_token[1]=='B') ret_wert=SPS_PORTB;
    if(AWL_FILE.awl_token[1]=='C') ret_wert=SPS_PORTC;
    if(AWL_FILE.awl_token[1]=='D') ret_wert=SPS_PORTD;
    if(AWL_FILE.awl_token[1]=='E') ret_wert=SPS_PORTE;
    if(AWL_FILE.awl_token[1]=='F') ret_wert=SPS_PORTF;
    if(AWL_FILE.awl_token[1]=='G') ret_wert=SPS_PORTG;

    if(AWL_FILE.awl_token[2]=='0') ret_wert|=SPS_BIT0;
    if(AWL_FILE.awl_token[2]=='1') ret_wert|=SPS_BIT1;
    if(AWL_FILE.awl_token[2]=='2') ret_wert|=SPS_BIT2;
    if(AWL_FILE.awl_token[2]=='3') ret_wert|=SPS_BIT3;
    if(AWL_FILE.awl_token[2]=='4') ret_wert|=SPS_BIT4;
    if(AWL_FILE.awl_token[2]=='5') ret_wert|=SPS_BIT5;
    if(AWL_FILE.awl_token[2]=='6') ret_wert|=SPS_BIT6;
    if(AWL_FILE.awl_token[2]=='7') ret_wert|=SPS_BIT7;
    if(AWL_FILE.awl_token[2]=='8') ret_wert|=SPS_BIT8;
    if(AWL_FILE.awl_token[2]=='9') ret_wert|=SPS_BIT9;

    if(AWL_FILE.awl_token[0]!='P') ret_wert=0;
  }
  else if(l==4) {
    // z.B. PB13
    if(AWL_FILE.awl_token[1]=='A') ret_wert=SPS_PORTA;
    if(AWL_FILE.awl_token[1]=='B') ret_wert=SPS_PORTB;
    if(AWL_FILE.awl_token[1]=='C') ret_wert=SPS_PORTC;
    if(AWL_FILE.awl_token[1]=='D') ret_wert=SPS_PORTD;
    if(AWL_FILE.awl_token[1]=='E') ret_wert=SPS_PORTE;
    if(AWL_FILE.awl_token[1]=='F') ret_wert=SPS_PORTF;
    if(AWL_FILE.awl_token[1]=='G') ret_wert=SPS_PORTG;

    if(AWL_FILE.awl_token[3]=='0') ret_wert|=SPS_BIT10;
    if(AWL_FILE.awl_token[3]=='1') ret_wert|=SPS_BIT11;
    if(AWL_FILE.awl_token[3]=='2') ret_wert|=SPS_BIT12;
    if(AWL_FILE.awl_token[3]=='3') ret_wert|=SPS_BIT13;
    if(AWL_FILE.awl_token[3]=='4') ret_wert|=SPS_BIT14;
    if(AWL_FILE.awl_token[3]=='5') ret_wert|=SPS_BIT15;

    if(AWL_FILE.awl_token[0]!='P') ret_wert=0;
    if(AWL_FILE.awl_token[2]!='1') ret_wert=0;
  }

  if(((ret_wert&0xFF00)==0) || ((ret_wert&0x00FF)==0)) {
    ret_wert=0;
    Parser_setError(AWL_ERR_UNKNOWN_PORTPIN);
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// Auswertung vom ADC-Pin Token
// ret_wert : 0 = error
//            [PA0] = 0x0101 bis [PA7] = 0x0108
//            [PB0] = 0x0201 bis [PB1] = 0x0202
//            [PC0] = 0x0301 bis [PC5] = 0x0306
//--------------------------------------------------------------
uint32_t checkAdcPin(void)
{
  uint32_t ret_wert=0;
  uint32_t l;

  l=getNextToken();
  if(l==3) {
    // z.B. PA3
    if(AWL_FILE.awl_token[1]=='A') ret_wert=SPS_PORTA;
    if(AWL_FILE.awl_token[1]=='B') ret_wert=SPS_PORTB;
    if(AWL_FILE.awl_token[1]=='C') ret_wert=SPS_PORTC;

    if(AWL_FILE.awl_token[2]=='0') ret_wert|=SPS_BIT0;
    if(AWL_FILE.awl_token[2]=='1') ret_wert|=SPS_BIT1;
    if(AWL_FILE.awl_token[2]=='2') ret_wert|=SPS_BIT2;
    if(AWL_FILE.awl_token[2]=='3') ret_wert|=SPS_BIT3;
    if(AWL_FILE.awl_token[2]=='4') ret_wert|=SPS_BIT4;
    if(AWL_FILE.awl_token[2]=='5') ret_wert|=SPS_BIT5;
    if(AWL_FILE.awl_token[2]=='6') ret_wert|=SPS_BIT6;
    if(AWL_FILE.awl_token[2]=='7') ret_wert|=SPS_BIT7;

    if(AWL_FILE.awl_token[0]!='P') ret_wert=0;
  }

  // check ob Pin = ADC-Pin
  // Port-A (Bit 0 bis 7)
  if((ret_wert&0xFF00)==SPS_PORTB) {
    // Port-B (Bit 0 bis 1)
    if((ret_wert&0x00FF)==SPS_BIT2) ret_wert=0;
    if((ret_wert&0x00FF)==SPS_BIT3) ret_wert=0;
    if((ret_wert&0x00FF)==SPS_BIT4) ret_wert=0;
    if((ret_wert&0x00FF)==SPS_BIT5) ret_wert=0;
    if((ret_wert&0x00FF)==SPS_BIT6) ret_wert=0;
    if((ret_wert&0x00FF)==SPS_BIT7) ret_wert=0;
  }
  if((ret_wert&0xFF00)==SPS_PORTC) {
    // Port-C (Bit 0 bis 5)
    if((ret_wert&0x00FF)==SPS_BIT6) ret_wert=0;
    if((ret_wert&0x00FF)==SPS_BIT7) ret_wert=0;
  }

  if(((ret_wert&0xFF00)==0) || ((ret_wert&0x00FF)==0)) {
    ret_wert=0;
    Parser_setError(AWL_ERR_UNKNOWN_PORTPIN);
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// Auswertung vom VarTyp Token
// ret_wert : SPS_TYP_NONE = error
//            !=SPS_TYP_NONE = Variablen Typ
//--------------------------------------------------------------
uint32_t checkVarTyp(void)
{
  uint32_t ret_wert=SPS_TYP_NONE;
  uint32_t l;

  l=getNextToken();
  if(l>0) {
    if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_TYP_BOOL)==0) {
      ret_wert=SPS_TYP_BOOL;
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_TYP_WORD)==0) {
      ret_wert=SPS_TYP_WORD;
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_TYP_COUNTER)==0) {
      ret_wert=SPS_TYP_COUNTER;
    }
    else if(strcmp(AWL_FILE.awl_token,AWL_TOKEN_TYP_TIMER)==0) {
      ret_wert=SPS_TYP_TIMER;
    }
  }

  if(ret_wert==SPS_TYP_NONE) {
    Parser_setError(AWL_ERR_UNKNOWN_VARTYP);
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// sucht die variable "AWL_FILE.awl_token"
// in der VAR_IN, VAR_OUT, VAR_MEM Liste
// ret_wert : AWL_LIST_NONE = variable nicht gefunden
//            != AWL_LIST_NONE = Variablen gefunden (liste, nr)
//--------------------------------------------------------------
uint32_t Parser_searchBitVariable(uint16_t* liste, uint16_t* nr)
{
  uint32_t ret_wert=AWL_LIST_NONE;
  uint32_t l,n;

  l=getNextToken();
  if(l>0) {
    if(VAR_INPUT.anz>0) {
      for(n=0;n<VAR_INPUT.anz;n++) {
        if(strcmp(AWL_FILE.awl_token,VAR_INPUT.token[n])==0) {
          *liste=AWL_LIST_INPUT;
          *nr=n;
          ret_wert=AWL_LIST_INPUT;
          break;
        }
      }
    }

    if(ret_wert!=AWL_LIST_NONE) return(ret_wert);

    if(VAR_OUTPUT.anz>0) {
      for(n=0;n<VAR_OUTPUT.anz;n++) {
        if(strcmp(AWL_FILE.awl_token,VAR_OUTPUT.token[n])==0) {
          *liste=AWL_LIST_OUTPUT;
          *nr=n;
          ret_wert=AWL_LIST_OUTPUT;
          break;
        }
      }
    }

    if(ret_wert!=AWL_LIST_NONE) return(ret_wert);

    if(VAR_MEMORY.anz>0) {
      for(n=0;n<VAR_MEMORY.anz;n++) {
        if(strcmp(AWL_FILE.awl_token,VAR_MEMORY.token[n])==0) {
          *liste=AWL_LIST_MEMORY;
          *nr=n;
          ret_wert=AWL_LIST_MEMORY;
          break;
        }
      }
    }

    if(ret_wert!=AWL_LIST_NONE) return(ret_wert);

  }

  if(ret_wert==AWL_LIST_NONE) {
    *liste=0;
    *nr=0;
    Parser_setError(AWL_ERR_UNKNOWN_VARIABLE);
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// sucht die variable "AWL_FILE.awl_token"
// in der VAR_MEM,VAR_ADC Liste
// ret_wert : AWL_LIST_NONE = variable nicht gefunden
//            AWL_LIST_MEMORY = Variable gefunden (liste, nr)
//            AWL_LIST_ADC    = Variable gefunden (liste, nr)
//            AWL_LIST_CONSTANT = variable = zahl (0, wert)
//--------------------------------------------------------------
uint32_t Parser_searchVariableOrInteger(uint16_t* liste, uint16_t* nr)
{
  uint32_t ret_wert=AWL_LIST_NONE;
  uint32_t l,n;
  int wert;

  l=getNextToken();
  if(l>0) {
    if(VAR_MEMORY.anz>0) {
      for(n=0;n<VAR_MEMORY.anz;n++) {
        if(strcmp(AWL_FILE.awl_token,VAR_MEMORY.token[n])==0) {
          *liste=AWL_LIST_MEMORY;
          *nr=n;
          ret_wert=AWL_LIST_MEMORY;
          break;
        }
      }
    }

    if(ret_wert!=AWL_LIST_NONE) return(ret_wert);

    if(VAR_ADC.anz>0) {
      for(n=0;n<VAR_ADC.anz;n++) {
        if(strcmp(AWL_FILE.awl_token,VAR_ADC.token[n])==0) {
          *liste=AWL_LIST_ADC;
          *nr=n;
          ret_wert=AWL_LIST_ADC;
          break;
        }
      }
    }

    if(ret_wert!=AWL_LIST_NONE) return(ret_wert);

    wert=dezStringToInt(l);
    if(wert>=0) {
      ret_wert=AWL_LIST_CONSTANT;
      *liste=AWL_LIST_NONE;
      *nr=(uint16_t)(wert);
    }
  }

  if(ret_wert==AWL_LIST_NONE) {
    *liste=0;
    *nr=0;
    Parser_setError(AWL_ERR_UNKNOWN_VARIABLE);
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// wandelt "AWL_FILE.awl_token" in eine Zahl
//  "123" = 123
//
// rert_wert :   0...65535 = Ergebnis
//             < 0         = Fehler
//--------------------------------------------------------------
int32_t dezStringToInt(uint32_t len)
{
  int32_t ret_wert=0;
  uint32_t n;
  uint8_t c;

  if((len>0) && (len<6)) {
    for(n=0;n<len;n++) {
      c=AWL_FILE.awl_token[n];
      if((c<48) || (c>57)) ret_wert=-2; // nur 0 bis 9
    }
    if(ret_wert!=0) return(ret_wert);

    ret_wert=0;
    for(n=0;n<len;n++) {
      ret_wert*=10;
      c=AWL_FILE.awl_token[n];
      ret_wert+=(c-48);
    }

    if(ret_wert>65535) ret_wert=-3; // nur 16bit
  }
  else {
    ret_wert=-1;
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// sucht die variable "AWL_FILE.awl_token"
// in der VAR_MEM Liste
// ret_wert : AWL_LIST_NONE  = variable nicht gefunden
//            AWL_LIST_MEMORY  = Variable gefunden (liste, nr)
//--------------------------------------------------------------
uint32_t Parser_searchVariable(uint16_t* liste, uint16_t* nr)
{
  uint32_t ret_wert=AWL_LIST_NONE;
  uint32_t l,n;

  l=getNextToken();
  if(l>0) {
    if(VAR_MEMORY.anz>0) {
      for(n=0;n<VAR_MEMORY.anz;n++) {
        if(strcmp(AWL_FILE.awl_token,VAR_MEMORY.token[n])==0) {
          *liste=AWL_LIST_MEMORY;
          *nr=n;
          ret_wert=AWL_LIST_MEMORY;
          break;
        }
      }
    }

    if(ret_wert!=AWL_LIST_NONE) return(ret_wert);

  }

  if(ret_wert==AWL_LIST_NONE) {
    *liste=0;
    *nr=0;
    Parser_setError(AWL_ERR_UNKNOWN_VARIABLE);
  }

  return(ret_wert);
}
