//--------------------------------------------------------------
// File     : sps_parser.h
//--------------------------------------------------------------

//--------------------------------------------------------------
#ifndef __STM32F4_UB_SPS_PARSER_H
#define __STM32F4_UB_SPS_PARSER_H



//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32f4xx.h"
#include "sps_parser_oc.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>



//--------------------------------------------------------------
// Rahmenbedingungen fuer die AWL
//--------------------------------------------------------------
#define  AWL_MAX_TOKEN_LEN            20 // max 20 zeichen pro Token
#define  AWL_MAX_VAR_IN               20 // max 20 input variabeln
#define  AWL_MAX_VAR_OUT              20 // max 20 output variabeln
#define  AWL_MAX_VAR_MEMORY           50 // max 50 memory variabeln
#define  AWL_MAX_VAR_ADC              16 // max 16 adc variabeln
#define  SPS_MAX_SIZE                500 // max 500 Opcodes
#define  AWL_MAX_SIZE               5000 // max 5000 Bytes
#define  AWL_MAX_STACK                10 // max 10 Verzweigungen


//--------------------------------------------------------------
// AWL Puffer im RAM
//--------------------------------------------------------------
uint8_t AWL_BUFFER[AWL_MAX_SIZE];


//--------------------------------------------------------------
// Paser Error-List
//--------------------------------------------------------------
#define  AWL_ERR_NO_ERROR            0x00 // alles OK
#define  AWL_ERR_FILESIZE_0          0x01 // File hat 0 Bytes
#define  AWL_ERR_NO_ASCII            0x02 // Unerlaubtes Zeichen
#define  AWL_ERR_TOKEN_LEN           0x03 // Token zu lang
#define  AWL_ERR_UNKNOWN_TOKEN       0x04 // Token unbekannt
#define  AWL_ERR_UNKNOWN_PORTPIN     0x05 // Portpin unbekannt
#define  AWL_ERR_UNKNOWN_VARTYP      0x06 // Variablentyp unbekannt
#define  AWL_ERR_AT_MISSING          0x07 // "AT" fehlt
#define  AWL_ERR_DP_MISSING          0x08 // ":" fehlt
#define  AWL_ERR_VAR_OUT_MISSING     0x09 // keinen OUT-Pin deklariert
#define  AWL_ERR_START_MISSING       0x0A // "AWL_START" fehlt
#define  AWL_ERR_UNKNOWN_VARIABLE    0x0B // unbekannte Variable
#define  AWL_ERR_FLAG_ERROR          0x0C // Flag error
#define  AWL_ERR_FILESIZE_MAX        0x0D // File zu gross
#define  AWL_ERR_OPCODE_MAX          0x0E // max anzahl Opcodes erreicht
#define  AWL_ERR_VAR_IN_MAX          0x0F // max anzahl VAR_IN erreicht
#define  AWL_ERR_VAR_OUT_MAX         0x10 // max anzahl VAR_OUT erreicht
#define  AWL_ERR_VAR_MEM_MAX         0x11 // max anzahl VAR erreicht
#define  AWL_ERR_VAR_ADC_MAX         0x12 // max anzahl VAR_ADC erreicht
#define  AWL_ERR_STACK_OVF           0x13 // Stack ueberlauf
#define  AWL_ERR_STACK_UNF           0x14 // Stack unterlauf
#define  AWL_ERR_STACK_UNEQUAL       0x15 // Stack != 0
#define  AWL_ERR_WRONG_VARTYP        0x16 // unerlaubter Variablentyp


//--------------------------------------------------------------
// liste aller Token im AWL-File
//
// E=phys. Eingang  , A=phys. Ausgang,  ADC=ADC Eingang
// MB=boolsche Variable , MW Word Variable
// T=Timer   , Z=Zehler
//--------------------------------------------------------------
// Allgemeine Token
#define  AWL_TOKEN_REM        "//"
#define  AWL_TOKEN_VAR_INPUT  "VAR_INPUT"
#define  AWL_TOKEN_VAR_OUTPUT "VAR_OUTPUT"
#define  AWL_TOKEN_VAR_MEMORY "VAR"
#define  AWL_TOKEN_VAR_ADC    "VAR_ADC"
#define  AWL_TOKEN_VAR_END    "END_VAR"
#define  AWL_TOKEN_VAR_AT     "AT"
#define  AWL_TOKEN_VAR_DP     ":"
#define  AWL_TOKEN_AWL_START  "AWL_START"
#define  AWL_TOKEN_AWL_END    "END_AWL"
// Token der Variabeln Typen
#define  AWL_TOKEN_TYP_BOOL    "BOOL"
#define  AWL_TOKEN_TYP_WORD    "WORD"
#define  AWL_TOKEN_TYP_COUNTER "COUNTER"
#define  AWL_TOKEN_TYP_TIMER   "TIMER"
// Token fr Logische Verknpfungen
#define  AWL_TOKEN_U          "U"   // "U <bit>"  [E,A,MB,MW,T,Z]
#define  AWL_TOKEN_UK         "U("  // "U("
#define  AWL_TOKEN_UN         "UN"  // "UN <bit>" [E,A,MB,MW,T,Z]
#define  AWL_TOKEN_UNK        "// UN("
#define  AWL_TOKEN_O          "O"   // "O <bit>"  [E,A,MB,MW,T,Z]
#define  AWL_TOKEN_OK         "O("  // "O("
#define  AWL_TOKEN_ON         "ON"  // "ON <bit>" [E,A,MB,MW,T,Z]
#define  AWL_TOKEN_ONK        "// ON("
#define  AWL_TOKEN_X          "X"   // "X <bit>"  [E,A,MB,MW,T,Z]
#define  AWL_TOKEN_XK         "X("  // "X("
#define  AWL_TOKEN_XN         "XN"  // "XN <bit>" [E,A,MB,MW,T,Z]
#define  AWL_TOKEN_XNK        "// XN("
#define  AWL_TOKEN_KE         ")"   // ")"
#define  AWL_TOKEN_EQ         "="   // "= <bit>"  [E,A,MB]
// Set/Reset Token
#define  AWL_TOKEN_R          "R"   // "R <bit>"  [E,A,MB,T,Z]
#define  AWL_TOKEN_S          "S"   // "S <bit>"  [E,A,MB,Z]
// VKE Token
#define  AWL_TOKEN_NOT        "NOT" // "NOT"
#define  AWL_TOKEN_SET        "SET" // "SET"
#define  AWL_TOKEN_CLR        "CLR" // "CLR"
// Flanken Token
#define  AWL_TOKEN_FN         "FN"  // "FN <bit>" [E,A,MB]
#define  AWL_TOKEN_FP         "FP"  // "FP <bit>" [E,A,MB]
// Vergleichs Token
#define  AWL_TOKEN_VEQ        "==I" // "==I"
#define  AWL_TOKEN_VGK        "<>I" // "==I"
#define  AWL_TOKEN_VG         ">I"  // ">I"
#define  AWL_TOKEN_VK         "<I"  // "<I"
#define  AWL_TOKEN_VGG        ">=I" // ">=I"
#define  AWL_TOKEN_VKG        "<=I" // "<=I"
// Token fuer Berrechnungen
#define  AWL_TOKEN_RA         "+I"  // "+I"
#define  AWL_TOKEN_RS         "-I"  // "-I"
#define  AWL_TOKEN_RM         "*I"  // "*I"
#define  AWL_TOKEN_RD         "/I"  // "/I"
// Load/Transfer Token
#define  AWL_TOKEN_L          "L"   // "L 123" oder "L <word>" [MW,T,Z,ADC]
#define  AWL_TOKEN_T          "T"   // "T <word>" [MW]
// Token fr Counter
#define  AWL_TOKEN_ZV         "ZV"  // "ZV <word>" [Z]
#define  AWL_TOKEN_ZR         "ZR"  // "ZR <word>" [Z]
// Token fr Timer
#define  AWL_TOKEN_SI         "SI"  // "SI <word>" [T]
#define  AWL_TOKEN_SV         "SV"  // "SV <word>" [T]
#define  AWL_TOKEN_SE         "SE"  // "SE <word>" [T]
#define  AWL_TOKEN_SS         "SS"  // "SS <word>" [T]
#define  AWL_TOKEN_SA         "SA"  // "SA <word>" [T]



//--------------------------------------------------------------
// liste aller Opcodes fuer den Interpreter
//--------------------------------------------------------------
// Opcodes fr Logische Verknpfungen
#define  AWL_OPCODE_NOP     0x0000
#define  AWL_OPCODE_U       0x0001
#define  AWL_OPCODE_UK      0x0002
#define  AWL_OPCODE_UN      0x0003
#define  AWL_OPCODE_UNK     0x0004
#define  AWL_OPCODE_O       0x0005
#define  AWL_OPCODE_OK      0x0006
#define  AWL_OPCODE_ON      0x0007
#define  AWL_OPCODE_ONK     0x0008
#define  AWL_OPCODE_X       0x0009
#define  AWL_OPCODE_XK      0x000A
#define  AWL_OPCODE_XN      0x000B
#define  AWL_OPCODE_XNK     0x000C
#define  AWL_OPCODE_KE      0x000D
#define  AWL_OPCODE_EQ      0x000E
// Set/Reset Opcodes
#define  AWL_OPCODE_R       0x000F
#define  AWL_OPCODE_S       0x0010
// VKE Opcodes
#define  AWL_OPCODE_NOT     0x0011
#define  AWL_OPCODE_SET     0x0012
#define  AWL_OPCODE_CLR     0x0013
// Flanken Opcodes
#define  AWL_OPCODE_FN      0x0014
#define  AWL_OPCODE_FP      0x0015
// Vergleichs Opcodes
#define  AWL_OPCODE_VEQ     0x0016
#define  AWL_OPCODE_VGK     0x0017
#define  AWL_OPCODE_VG      0x0018
#define  AWL_OPCODE_VK      0x0019
#define  AWL_OPCODE_VGG     0x001A
#define  AWL_OPCODE_VKG     0x001B
// Opcodes fuer Berrechnungen
#define  AW_OPCODE_RA       0x001C
#define  AW_OPCODE_RS       0x001D
#define  AW_OPCODE_RM       0x001E
#define  AW_OPCODE_RD       0x001F
// Load/Transfer Opcodes
#define  AWL_OPCODE_LM      0x0020
#define  AWL_OPCODE_LC      0x0021
#define  AWL_OPCODE_T       0x0022
// Opcodes fr Counter
#define  AWL_OPCODE_ZV      0x0023
#define  AWL_OPCODE_ZR      0x0024
// Opcodes fr Timer
#define  AWL_OPCODE_SI      0x0025
#define  AWL_OPCODE_SV      0x0026
#define  AWL_OPCODE_SE      0x0027
#define  AWL_OPCODE_SS      0x0028
#define  AWL_OPCODE_SA      0x0029


//--------------------------------------------------------------
// Defines der Listen
//--------------------------------------------------------------
#define  AWL_LIST_NONE      0x0000
#define  AWL_LIST_INPUT     0x0001  // VAR_INPUT
#define  AWL_LIST_OUTPUT    0x0002  // VAR_OUTPUT
#define  AWL_LIST_MEMORY    0x0003  // VAR
#define  AWL_LIST_ADC       0x0004  // VAR_ADC
#define  AWL_LIST_CONSTANT  0x0005  // sonder fuer Konstante Zahl


//--------------------------------------------------------------
// Defines der Flags
//--------------------------------------------------------------
#define  AWL_FLAG_ERR           0x0000
#define  AWL_FLAG_CHECK_OK      0x0001 // alle Zeichen=Ascii
#define  AWL_FLAG_VAR_OUT       0x0002 // OUT ist definiert
#define  AWL_FLAG_START         0x0004 // AWL_START ist definiert
#define  AWL_FLAG_END           0x0008 // AWL_END ist definiert
#define  AWL_FLAG_PARSE_OK      0x0010 // Parsing ohne Fehler
#define  AWL_FLAG_PRG_OK        0x0020 // SPS File > 0
#define  AWL_FLAG_ALL_OK        0x003F // alle Flags auf Hi



//--------------------------------------------------------------
// Defines der GPIO
//--------------------------------------------------------------
#define   SPS_PORT_NONE         0x0000
#define   SPS_PORTA             0x0100
#define   SPS_PORTB             0x0200
#define   SPS_PORTC             0x0300
#define   SPS_PORTD             0x0400
#define   SPS_PORTE             0x0500
#define   SPS_PORTF             0x0600
#define   SPS_PORTG             0x0700

#define   SPS_BIT_NONE          0x0000
#define   SPS_BIT0              0x0001
#define   SPS_BIT1              0x0002
#define   SPS_BIT2              0x0003
#define   SPS_BIT3              0x0004
#define   SPS_BIT4              0x0005
#define   SPS_BIT5              0x0006
#define   SPS_BIT6              0x0007
#define   SPS_BIT7              0x0008
#define   SPS_BIT8              0x0009
#define   SPS_BIT9              0x000A
#define   SPS_BIT10             0x000B
#define   SPS_BIT11             0x000C
#define   SPS_BIT12             0x000D
#define   SPS_BIT13             0x000E
#define   SPS_BIT14             0x000F
#define   SPS_BIT15             0x0010


//--------------------------------------------------------------
// Defines der Typen (bei Liste=VAR)
//--------------------------------------------------------------
#define   SPS_TYP_NONE          0x00
#define   SPS_TYP_BOOL          0x01 // 1bit Variable
#define   SPS_TYP_WORD          0x02 // 16bit Variable
#define   SPS_TYP_COUNTER       0x03 // 16bit Counter
#define   SPS_TYP_TIMER         0x04 // 16bit Timer


//--------------------------------------------------------------
// Defines fr Status (bei Typ=Timer)
//--------------------------------------------------------------
#define   SPS_TIM_STATUS_STOP     0x00 // Timer ist AUS
#define   SPS_TIM_STATUS_START    0x01 // Timer luft
#define   SPS_TIM_STATUS_TIMEOUT  0x02 // Zeit ist abgelaufen


//--------------------------------------------------------------
// Struktur von einem AWL-File
//--------------------------------------------------------------
typedef struct UB_AWL_t
{
  const uint8_t *table; // Tabelle mit den Daten
  uint32_t size;        // Anzahl der Bytes
}UB_AWL;

// Globale AWL-Struktur
UB_AWL awl_struct;


//--------------------------------------------------------------
// Globale AWL-Daten
//--------------------------------------------------------------
typedef struct {
  uint32_t awl_quelle;    // 0=FLASH, 1=RAM
  uint32_t uart_mode;     // 0=normal, 1=load
  uint32_t awl_ram_size;
  uint32_t awl_flash_size;
}AWL_Global_t;
AWL_Global_t AWL_GLOBAL;


//--------------------------------------------------------------
// Error Struktur
//--------------------------------------------------------------
typedef struct {
  uint32_t err_nr;
  uint32_t err_line;
  char err_token[AWL_MAX_TOKEN_LEN];
}AWL_Error_t;
AWL_Error_t AWL_ERR;


//--------------------------------------------------------------
// Struktur fuer ein AWL Program
//--------------------------------------------------------------
typedef struct {
  uint32_t awl_akt_size;
  uint32_t awl_aktpos;
  uint32_t awl_akt_line;
  int16_t awl_akt_stack; // muss signed sein
  uint32_t awl_flags;
  uint32_t awl_abschnitt; // 0=Allgemein, 1=Programm
  char awl_token[AWL_MAX_TOKEN_LEN];
}AWL_File_t;
AWL_File_t AWL_FILE;


//--------------------------------------------------------------
// Struktur von einem GPIO Input (28 Bytes pro Eintrag)
//--------------------------------------------------------------
typedef struct {
  uint32_t anz;
  char token[AWL_MAX_TOKEN_LEN][AWL_MAX_VAR_IN];
  uint16_t address[AWL_MAX_VAR_IN];
  uint8_t bit_wert[AWL_MAX_VAR_IN];
  uint8_t old_bit_wert[AWL_MAX_VAR_IN];
}VAR_Input_t;
VAR_Input_t VAR_INPUT;


//--------------------------------------------------------------
// Struktur von einem GPIO Output (28 Bytes pro Eintrag)
//--------------------------------------------------------------
typedef struct {
  uint32_t anz;
  char token[AWL_MAX_TOKEN_LEN][AWL_MAX_VAR_OUT];
  uint16_t address[AWL_MAX_VAR_OUT];
  uint8_t bit_wert[AWL_MAX_VAR_OUT];
  uint8_t old_bit_wert[AWL_MAX_VAR_OUT];
}VAR_Output_t;
VAR_Output_t VAR_OUTPUT;


//--------------------------------------------------------------
// Struktur von einem VAR (31 Bytes pro Eintrag)
//--------------------------------------------------------------
typedef struct {
  uint32_t anz;
  char token[AWL_MAX_TOKEN_LEN][AWL_MAX_VAR_MEMORY];
  uint8_t typ[AWL_MAX_VAR_MEMORY];
  uint8_t bit_wert[AWL_MAX_VAR_MEMORY];
  uint8_t old_bit_wert[AWL_MAX_VAR_MEMORY];
  uint16_t word_wert[AWL_MAX_VAR_MEMORY];
  uint8_t timer_bit_wert[AWL_MAX_VAR_MEMORY];
  uint8_t status[AWL_MAX_VAR_MEMORY];
}VAR_Memory_t;
VAR_Memory_t VAR_MEMORY;


//--------------------------------------------------------------
// Struktur von einem ADC Input (29 Bytes pro Eintrag)
//--------------------------------------------------------------
typedef struct {
  uint32_t anz;
  char token[AWL_MAX_TOKEN_LEN][AWL_MAX_VAR_ADC];
  uint16_t address[AWL_MAX_VAR_ADC];
  uint16_t word_wert[AWL_MAX_VAR_ADC];
}VAR_ADC_t;
VAR_ADC_t VAR_ADC;


//--------------------------------------------------------------
// Globale SPS-Daten
//--------------------------------------------------------------
typedef struct {
  uint32_t ist_cycluszeit_us;
  uint32_t soll_cycluszeit_ms;
  uint32_t ms_cnt;
}SPS_Global_t;
SPS_Global_t SPS_GLOBAL;


//--------------------------------------------------------------
// Struktur von einem SPS Programm
//--------------------------------------------------------------
typedef struct {
  uint32_t sps_size;
  uint32_t sps_aktpos;
  uint16_t bytecode[SPS_MAX_SIZE+10];
}SPS_Prog_t;
SPS_Prog_t SPS_PROG;



//--------------------------------------------------------------
// Globale Funktionen
//--------------------------------------------------------------
uint32_t Parser_Start(void);
void Parser_setError(uint32_t error);
uint32_t Parser_searchBitVariable(uint16_t* liste, uint16_t* nr);
uint32_t Parser_searchVariableOrInteger(uint16_t* liste, uint16_t* nr);
uint32_t Parser_searchVariable(uint16_t* liste, uint16_t* nr);


//--------------------------------------------------------------
#endif // __STM32F4_UB_SPS_PARSER_H
