//--------------------------------------------------------------
// File     : sps_parser_oc.c
// Datum    : 26.02.2014
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F4
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : SPS_Parser
// Funktion : SPS-Parser (Ausfhrung)
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "sps_parser_oc.h"




//--------------------------------------------------------------
// logischer Opcode
// Token1 = CMD, Token2 = VAR
// Byte1=Opcode, Byte2=Liste, Byte3=Nummer
//--------------------------------------------------------------
void parser_OC_Logic(uint16_t opcode)
{
  uint32_t wert;
  uint16_t liste,nr;

  SPS_PROG.bytecode[SPS_PROG.sps_size]=opcode;
  SPS_PROG.sps_size++;

  wert=Parser_searchBitVariable(&liste, &nr);
  if(wert!=AWL_LIST_NONE) {
    SPS_PROG.bytecode[SPS_PROG.sps_size]=liste;
    SPS_PROG.sps_size++;
    SPS_PROG.bytecode[SPS_PROG.sps_size]=nr;
    SPS_PROG.sps_size++;
  }
}


//--------------------------------------------------------------
// Zuweisung
// Token1 = CMD, Token2 = VAR
// Byte1=Opcode, Byte2=Liste, Byte3=Nummer
//--------------------------------------------------------------
void parser_OC_EQUAL(void)
{
  uint32_t wert;
  uint16_t liste,nr;
  uint8_t typ;

  SPS_PROG.bytecode[SPS_PROG.sps_size]=AWL_OPCODE_EQ;
  SPS_PROG.sps_size++;

  wert=Parser_searchBitVariable(&liste, &nr);
  if((wert==AWL_LIST_INPUT) || (wert==AWL_LIST_OUTPUT)) {
    SPS_PROG.bytecode[SPS_PROG.sps_size]=liste;
    SPS_PROG.sps_size++;
    SPS_PROG.bytecode[SPS_PROG.sps_size]=nr;
    SPS_PROG.sps_size++;
  }
  else if(wert==AWL_LIST_MEMORY) {
    typ=VAR_MEMORY.typ[nr];
    if(typ==SPS_TYP_BOOL) {
      SPS_PROG.bytecode[SPS_PROG.sps_size]=liste;
      SPS_PROG.sps_size++;
      SPS_PROG.bytecode[SPS_PROG.sps_size]=nr;
      SPS_PROG.sps_size++;
    }
    else {
      Parser_setError(AWL_ERR_WRONG_VARTYP);
    }
  }
}


//--------------------------------------------------------------
// logische Verzweigung (Auf)
// Token1 = CMD
// Byte1=Opcode
//--------------------------------------------------------------
void parser_OC_Logic_Verzweigung_On(uint16_t opcode)
{
  SPS_PROG.bytecode[SPS_PROG.sps_size]=opcode;
  SPS_PROG.sps_size++;

  AWL_FILE.awl_akt_stack++;
  if(AWL_FILE.awl_akt_stack>AWL_MAX_STACK) {
    Parser_setError(AWL_ERR_STACK_OVF);
  }
}


//--------------------------------------------------------------
// logische Verzweigung (Zu)
// Token1 = CMD
// Byte1=Opcode
//--------------------------------------------------------------
void parser_OC_Logic_Verzweigung_Off(uint16_t opcode)
{
  SPS_PROG.bytecode[SPS_PROG.sps_size]=opcode;
  SPS_PROG.sps_size++;

  AWL_FILE.awl_akt_stack--;
  if(AWL_FILE.awl_akt_stack<0) {
    Parser_setError(AWL_ERR_STACK_UNF);
  }
}


//--------------------------------------------------------------
// Reset, Set
// Token1 = CMD, Token2 = VAR
// Byte1=Opcode, Byte2=Liste, Byte3=Nummer
//--------------------------------------------------------------
void parser_OC_RS(uint16_t opcode)
{
  uint32_t wert;
  uint16_t liste,nr;
  uint8_t typ,ok=0;

  SPS_PROG.bytecode[SPS_PROG.sps_size]=opcode;
  SPS_PROG.sps_size++;

  wert=Parser_searchBitVariable(&liste, &nr);

  ok=0;
  if(wert==AWL_LIST_INPUT) ok=1;
  if(wert==AWL_LIST_OUTPUT) ok=1;
  if(wert==AWL_LIST_MEMORY) {
    typ=VAR_MEMORY.typ[nr];
    if(typ==SPS_TYP_BOOL) ok=1;
    if(typ==SPS_TYP_COUNTER) ok=1;
    if(opcode==AWL_OPCODE_R) {
      if(typ==SPS_TYP_TIMER) ok=1;
    }
  }

  if(ok==1) {
    SPS_PROG.bytecode[SPS_PROG.sps_size]=liste;
    SPS_PROG.sps_size++;
    SPS_PROG.bytecode[SPS_PROG.sps_size]=nr;
    SPS_PROG.sps_size++;
  }
  else {
    Parser_setError(AWL_ERR_WRONG_VARTYP);
  }
}


//--------------------------------------------------------------
// NOT, SET, CLR vom VKE
// Token1 = CMD
// Byte1 = Opcode
//--------------------------------------------------------------
void parser_OC_NSC(uint16_t opcode)
{
  SPS_PROG.bytecode[SPS_PROG.sps_size]=opcode;
  SPS_PROG.sps_size++;
}


//--------------------------------------------------------------
// Flanke
// Token1 = CMD, Token2 = VAR
// Byte1=Opcode, Byte2=Liste, Byte3=Nummer
//--------------------------------------------------------------
void parser_OC_Flanke(uint16_t opcode)
{
  uint32_t wert;
  uint16_t liste,nr;
  uint8_t typ;

  SPS_PROG.bytecode[SPS_PROG.sps_size]=opcode;
  SPS_PROG.sps_size++;

  wert=Parser_searchBitVariable(&liste, &nr);
  if((wert==AWL_LIST_INPUT) || (wert==AWL_LIST_OUTPUT)) {
    SPS_PROG.bytecode[SPS_PROG.sps_size]=liste;
    SPS_PROG.sps_size++;
    SPS_PROG.bytecode[SPS_PROG.sps_size]=nr;
    SPS_PROG.sps_size++;
  }
  else if(wert==AWL_LIST_MEMORY) {
    typ=VAR_MEMORY.typ[nr];
    if(typ==SPS_TYP_BOOL) {
      SPS_PROG.bytecode[SPS_PROG.sps_size]=liste;
      SPS_PROG.sps_size++;
      SPS_PROG.bytecode[SPS_PROG.sps_size]=nr;
      SPS_PROG.sps_size++;
    }
    else {
      Parser_setError(AWL_ERR_WRONG_VARTYP);
    }
  }
}


//--------------------------------------------------------------
// Vergleich
// Token1 = CMD
// Byte1=Opcode
//--------------------------------------------------------------
void parser_OC_Vergleich(uint16_t opcode)
{
  SPS_PROG.bytecode[SPS_PROG.sps_size]=opcode;
  SPS_PROG.sps_size++;
}


//--------------------------------------------------------------
// Rechne
// Token1 = CMD
// Byte1=Opcode
//--------------------------------------------------------------
void parser_OC_Rechne(uint16_t opcode)
{
  SPS_PROG.bytecode[SPS_PROG.sps_size]=opcode;
  SPS_PROG.sps_size++;
}


//--------------------------------------------------------------
// Load Befehl
// Token1 = CMD, Token2 = VAR/Konstante
// Memory    : Byte1=LM-Opcode, Byte2=Liste, Byte3=Nummer
// Konstante : Byte1=LC-Opcode, Byte2=Wert
//--------------------------------------------------------------
void parser_OC_Load(void)
{
  uint32_t wert;
  uint16_t liste,nr;
  uint8_t typ;

  wert=Parser_searchVariableOrInteger(&liste, &nr);
  if(wert==AWL_LIST_MEMORY) {
    typ=VAR_MEMORY.typ[nr];
    if((typ==SPS_TYP_COUNTER) || (typ==SPS_TYP_TIMER) ||
       (typ==SPS_TYP_WORD)) {
      SPS_PROG.bytecode[SPS_PROG.sps_size]=AWL_OPCODE_LM;
      SPS_PROG.sps_size++;
      SPS_PROG.bytecode[SPS_PROG.sps_size]=liste;
      SPS_PROG.sps_size++;
      SPS_PROG.bytecode[SPS_PROG.sps_size]=nr;
      SPS_PROG.sps_size++;
    }
    else {
      Parser_setError(AWL_ERR_WRONG_VARTYP);
    }
  }
  else if(wert==AWL_LIST_ADC) {
    SPS_PROG.bytecode[SPS_PROG.sps_size]=AWL_OPCODE_LM;
    SPS_PROG.sps_size++;
    SPS_PROG.bytecode[SPS_PROG.sps_size]=liste;
    SPS_PROG.sps_size++;
    SPS_PROG.bytecode[SPS_PROG.sps_size]=nr;
    SPS_PROG.sps_size++;
  }
  else if(wert==AWL_LIST_CONSTANT) {
    SPS_PROG.bytecode[SPS_PROG.sps_size]=AWL_OPCODE_LC;
    SPS_PROG.sps_size++;
    SPS_PROG.bytecode[SPS_PROG.sps_size]=nr;
    SPS_PROG.sps_size++;
  }
}


//--------------------------------------------------------------
// Transfer Befehl
// Token1 = CMD, Token2 = VAR
// Byte1=Opcode, Byte2=Liste, Byte3=Nummer
//--------------------------------------------------------------
void parser_OC_Transfer(void)
{
  uint32_t wert;
  uint16_t liste,nr;
  uint8_t typ;

  wert=Parser_searchVariable(&liste, &nr);
  if(wert==AWL_LIST_MEMORY) {
    typ=VAR_MEMORY.typ[nr];
    if(typ==SPS_TYP_WORD) {
      SPS_PROG.bytecode[SPS_PROG.sps_size]=AWL_OPCODE_T;
      SPS_PROG.sps_size++;
      SPS_PROG.bytecode[SPS_PROG.sps_size]=liste;
      SPS_PROG.sps_size++;
      SPS_PROG.bytecode[SPS_PROG.sps_size]=nr;
      SPS_PROG.sps_size++;
    }
    else {
      Parser_setError(AWL_ERR_WRONG_VARTYP);
    }
  }
}


//--------------------------------------------------------------
// Counter
// Token1 = CMD, Token2 = VAR
// Byte1=Opcode, Byte2=Liste, Byte3=Nummer
//--------------------------------------------------------------
void parser_OC_Counter(uint16_t opcode)
{
  uint32_t wert;
  uint16_t liste,nr;
  uint8_t typ;

  wert=Parser_searchVariable(&liste, &nr);
  if(wert==AWL_LIST_MEMORY) {
    typ=VAR_MEMORY.typ[nr];
    if(typ==SPS_TYP_COUNTER) {
      SPS_PROG.bytecode[SPS_PROG.sps_size]=opcode;
      SPS_PROG.sps_size++;
      SPS_PROG.bytecode[SPS_PROG.sps_size]=liste;
      SPS_PROG.sps_size++;
      SPS_PROG.bytecode[SPS_PROG.sps_size]=nr;
      SPS_PROG.sps_size++;
    }
    else {
      Parser_setError(AWL_ERR_WRONG_VARTYP);
    }
  }
}


//--------------------------------------------------------------
// Timer
// Token1 = CMD, Token2 = VAR
// Byte1=Opcode, Byte2=Liste, Byte3=Nummer
//--------------------------------------------------------------
void parser_OC_Timer(uint16_t opcode)
{
  uint32_t wert;
  uint16_t liste,nr;
  uint8_t typ;

  wert=Parser_searchVariable(&liste, &nr);
  if(wert==AWL_LIST_MEMORY) {
    typ=VAR_MEMORY.typ[nr];
    if(typ==SPS_TYP_TIMER) {
      SPS_PROG.bytecode[SPS_PROG.sps_size]=opcode;
      SPS_PROG.sps_size++;
      SPS_PROG.bytecode[SPS_PROG.sps_size]=liste;
      SPS_PROG.sps_size++;
      SPS_PROG.bytecode[SPS_PROG.sps_size]=nr;
      SPS_PROG.sps_size++;
    }
    else {
      Parser_setError(AWL_ERR_WRONG_VARTYP);
    }
  }
}
