//--------------------------------------------------------------
// File     : stm32_ub_sps.c
// Datum    : 26.02.2014
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F4
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : STM32_UB_UART,STM32_UB_Systick,
//            SPS_Parser, SPS_Interpreter
// Funktion : SPS-Funktionen
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_sps.h"



//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
void p_init_awl_global(uint8_t quelle, uint8_t uart_mode);
void p_send_parsing_debug(void);


//--------------------------------------------------------------
// init der SPS
//--------------------------------------------------------------
void UB_SPS_Init(void)
{
  // init vom UART
  UB_Uart_Init();

  // init vom Systick
  UB_Systick_Init();

  // Status Meldung per UART senden
  UB_SPS_sendStringUART(" ");
  UB_SPS_sendStringUART("SPS_ALIVE [V:1.00]");

  // AWL zuweisen
  awl_struct=AWL1;

  // init aller Globalen Variablen (flash, normal)
  p_init_awl_global(0,0);
}


//--------------------------------------------------------------
// start der SPS
// endloslauf (bis stop vom User oder Fehler)
//
// ret_wert :  0 = SPS vom User gestoppt
//            >0 = Fehlercode
//--------------------------------------------------------------
uint32_t UB_SPS_Start_AWL(void)
{
  uint32_t ret_wert=0;
  uint32_t parser_check;
  uint32_t interpreter_check;

  // check ob AWL von Flash oder RAM
  if(AWL_GLOBAL.awl_quelle==0) {
    UB_SPS_sendStringUART(" ");
    UB_SPS_sendStringUART("AWL from Flash");
  }
  else {
    UB_SPS_sendStringUART(" ");
    UB_SPS_sendStringUART("AWL from RAM");
  }

  // start vom Parser
  UB_SPS_sendStringUART(" ");
  UB_SPS_sendStringUART("start parser...");
  parser_check=Parser_Start();
  ret_wert=parser_check;
  // test ob Fehler beim Parser aufgetreten ist
  if(parser_check==0) {
    UB_SPS_sendStringUART("parsing ok");
    sprintf(str_buf,"bytecodes : %d",(int)(SPS_PROG.sps_size));
    UB_SPS_sendStringUART(str_buf);
  }
  else {
    UB_SPS_sendStringUART(" ");
    sprintf(str_buf,"parser errorcode : %02Xh",(int)(AWL_ERR.err_nr));
    UB_SPS_sendStringUART(str_buf);
    sprintf(str_buf,"error in line : %d",(int)(AWL_ERR.err_line));
    UB_SPS_sendStringUART(str_buf);
    sprintf(str_buf,"error token : %s",AWL_ERR.err_token);
    UB_SPS_sendStringUART(str_buf);
  }

  #if SPS_USE_PARSING_DEBUG==1
    p_send_parsing_debug();
  #endif

  // wenn parsing ok, interpreter starten
  if(parser_check==0) {
    // init vom Interpreter
    UB_SPS_sendStringUART(" ");
    UB_SPS_sendStringUART("init interpreter...");
    Interpreter_Init();
    // test vom Interpreter
    UB_SPS_sendStringUART("SPS test...");
    interpreter_check=Interpteter_Test();
    ret_wert=interpreter_check;
    // test ob fehler aufgetreten ist
    if(interpreter_check==0) {
      // gemessene Durchlaufzeit anzeigen
      sprintf(str_buf,"cylcetime : %d us",(int)(SPS_GLOBAL.ist_cycluszeit_us));
      UB_SPS_sendStringUART(str_buf);
      UB_SPS_sendStringUART(" ");
      UB_SPS_sendStringUART(SPS_AKTIV_STRING);
      sprintf(str_buf,"OK! SPS running at %d ms cylcetime",(int)(SPS_GLOBAL.soll_cycluszeit_ms));
      UB_SPS_sendStringUART(str_buf);
      UB_SPS_sendStringUART("send 'SPS_STOP' to terminate");
      // start vom Interpreter
      interpreter_check=Interpteter_Start();
      ret_wert=interpreter_check;

      // test ob SPS vom User gestoppt wurde
      if(interpreter_check==0) {
        UB_SPS_sendStringUART(" ");
        UB_SPS_sendStringUART("SPS stopped by user");
      }
    }

    // Ausgnge abschalten
    Interpreter_Stop();    

    // test ob fehler aufgetreten ist
    if(interpreter_check!=0) {
      UB_SPS_sendStringUART(" ");
      UB_SPS_sendStringUART("SPS stopped by error");
      sprintf(str_buf,"interpreter errorcode : %02Xh",(int)(SPS_FILE.err_nr));
      UB_SPS_sendStringUART(str_buf);
      sprintf(str_buf,"in opcode : %d",(int)(SPS_FILE.err_pos));
      UB_SPS_sendStringUART(str_buf);
    }
    UB_SPS_sendStringUART(SPS_INAKTIV_STRING);
  }
  else {
    UB_SPS_sendStringUART(" ");
    UB_SPS_sendStringUART("SPS not aktivated");
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// sendet einen String per UART
//--------------------------------------------------------------
void UB_SPS_sendStringUART(char *ptr)
{
  UB_Uart_SendString(COM3,ptr,CRLF);
}


//--------------------------------------------------------------
// test ob Abbruchsequenz per UART empfangen wurde
// ret_wert : 0 = nichts empfangen
//            1 = STOP-Kommando empfangen
//--------------------------------------------------------------
uint8_t UB_SPS_checkUART_ESC(void)
{
  uint8_t ret_wert=0;

  if(UB_Uart_ReceiveString(COM3,str_buf)==RX_READY) {
    if(strcmp(str_buf,SPS_ESC_STRING)==0) {
      ret_wert=1;
    }
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// init der globalen AWL Variabeln
// quelle : 0=flash, 1=ram
// mode   : 0=normal, 1=load
//--------------------------------------------------------------
void p_init_awl_global(uint8_t quelle, uint8_t uart_mode)
{
  AWL_GLOBAL.awl_quelle=quelle;    // 0=Flash, 1=RAM
  AWL_GLOBAL.uart_mode=uart_mode;  // 0=normal, 1=Load
  AWL_GLOBAL.awl_ram_size=0;
  AWL_GLOBAL.awl_flash_size=awl_struct.size;
}


//--------------------------------------------------------------
// sendet Debug Infos vom Parsing
//--------------------------------------------------------------
void p_send_parsing_debug(void)
{
#if SPS_USE_PARSING_DEBUG==1

  uint32_t n;

  UB_SPS_sendStringUART(" ");
  UB_SPS_sendStringUART("Parsing Debug Infos :");
  sprintf(str_buf,"AWL Filesize : %d Bytes",(int)(AWL_FILE.awl_akt_size));
  UB_SPS_sendStringUART(str_buf);
  sprintf(str_buf,"Status Flags : %02Xh ",(int)(AWL_FILE.awl_flags));
  UB_SPS_sendStringUART(str_buf);
  sprintf(str_buf,"VAR_INPUT : %d ",(int)(VAR_INPUT.anz));
  UB_SPS_sendStringUART(str_buf);
  sprintf(str_buf,"VAR_OUTPUT : %d ",(int)(VAR_OUTPUT.anz));
  UB_SPS_sendStringUART(str_buf);
  sprintf(str_buf,"VAR_MEMORY : %d ",(int)(VAR_MEMORY.anz));
  UB_SPS_sendStringUART(str_buf);
  sprintf(str_buf,"VAR_ADC : %d ",(int)(VAR_ADC.anz));
  UB_SPS_sendStringUART(str_buf);
  sprintf(str_buf,"SPS Bytecodes : %d ",(int)(SPS_PROG.sps_size));
  UB_SPS_sendStringUART(str_buf);

  for(n=0;n<SPS_PROG.sps_size;n++) {
    sprintf(str_buf,"OC[%d] = %02Xh",(int)(n),(int)(SPS_PROG.bytecode[n]));
    UB_SPS_sendStringUART(str_buf);
  }
#endif
}


//--------------------------------------------------------------
// laden von einem AWL-Textfile per UART
// ret_wert : 0=ok, >0=error
//--------------------------------------------------------------
uint32_t UB_SPS_loadAWL_UART(void)
{
  uint32_t ret_wert=0;
  uint32_t tim_cnt=0,timeout=0;
  uint32_t old_len=0;

  UB_SPS_sendStringUART(" ");
  UB_SPS_sendStringUART("ready for AWL-Transfer over UART");
  UB_SPS_sendStringUART("waiting...");

  // init der Globalen Variabeln (RAM, Load)
  AWL_ERR.err_nr=0;
  p_init_awl_global(1,1);

  // warten bis alle Bytes per UART in den SPS-Buffer geladen wurden
  timeout=800000; // ca 5sec timeout
  do {
    if(AWL_GLOBAL.awl_ram_size>0) tim_cnt++;
    if(AWL_GLOBAL.awl_ram_size!=old_len) {
      old_len=AWL_GLOBAL.awl_ram_size;
      tim_cnt=0;
      timeout=400000; // timeout verkrzen
    }
  }
  while(tim_cnt<timeout);

  // Status anzeigen
  if(AWL_ERR.err_nr==0) {
    sprintf(str_buf,"AWL loaded : %d Bytes", (int)(AWL_GLOBAL.awl_ram_size));
    UB_SPS_sendStringUART(str_buf);
  }
  else {
    UB_SPS_sendStringUART("error : AWL Filesize overflow");
  }
  
  // Load wieder deaktivieren
  AWL_GLOBAL.uart_mode=0;

  ret_wert=AWL_ERR.err_nr;

  return(ret_wert);
}
